<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 30/06/20
 * Time: 10.39
 */

namespace FiloBlu\ExpiredSessionRedirect\Plugin;

use FiloBlu\ExpiredSessionRedirect\Observer\SetCookie;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Stdlib\Cookie\CookieMetadataFactory;
use Magento\Framework\Stdlib\Cookie\PhpCookieManager;
use Psr\Log\LoggerInterface;
use Magento\Customer\Model\Session;

class RedirectPlugin
{

    /**
     * @var LoggerInterface
     */
    private $logger;
    /**
     * @var PhpCookieManager
     */
    private $cookieManager;
    /**
     * @var CookieMetadataFactory
     */
    private $cookieMetadataFactory;
    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * @var Session
     */
    private $session;

    /**
     * RedirectPlugin constructor.
     * @param LoggerInterface $logger
     * @param PhpCookieManager $cookieManager
     * @param CookieMetadataFactory $cookieMetadataFactory
     * @param ScopeConfigInterface $scopeConfig
     * @param Session $session
     */
    public function __construct(
        LoggerInterface $logger,
        PhpCookieManager $cookieManager,
        CookieMetadataFactory $cookieMetadataFactory,
        ScopeConfigInterface $scopeConfig,
        Session $session
    ) {
        $this->logger = $logger;
        $this->cookieManager = $cookieManager;
        $this->cookieMetadataFactory = $cookieMetadataFactory;
        $this->scopeConfig = $scopeConfig;
        $this->session = $session;
    }

    /**
     * @param \Magento\Framework\App\FrontControllerInterface $subject
     * @param RequestInterface $request
     */
    public function beforeDispatch(\Magento\Framework\App\FrontControllerInterface $subject, RequestInterface $request)
    {
        try {
            if ($this->scopeConfig->getValue(SetCookie::EXPIRED_SESSION_REDIRECT_ENABLE, \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE) ?? false) {
                $customer = $this->session->getCustomer();
                $customerHash = '';
                if ($customer && $customer->getId()) {
                    $customerHash = password_hash($customer->getId(), PASSWORD_DEFAULT);
                }
                if ($this->cookieManager->getCookie(SetCookie::COOKIE_NAME) !== null && !password_verify($this->cookieManager->getCookie(SetCookie::COOKIE_NAME), $customerHash)) {
                    //redirect
                    $metadata = $this->cookieMetadataFactory->createPublicCookieMetadata()->setPath('/');
                    $this->cookieManager->deleteCookie(SetCookie::COOKIE_NAME, $metadata);
                    if($request->getRequestString() != '/customer/account/logout/')
                    {
                        $url = $this->scopeConfig->getValue(SetCookie::EXPIRED_SESSION_REDIRECT_URL, \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE) ?? '';
                        if ($url != "") {
                            $request->setPathInfo($url);
                        }
                    }

                }
            }
        } catch (\Exception $e) {
            $this->logger->critical($e->getMessage());
        } catch (\Throwable $t) {
            $this->logger->critical($t->getMessage());
        }
    }
}
